/////////////////////////////////////////////////////////////////
//Dynamisches System Kapitel 10 Fraktale
//////////////////////////////////////////////////////////////

//Winkelzunahme alpha (im Bogenmass) pro Iteration
float alpha = 1.111;

//Bildausschnitt zu Beginn:
float xU =-1; 
float xO = 5;
float yU = -2; 
float yO = 2;

//Für starkes Zoomen müssen sehr viele Iterationen durchgeführt werden.
//Damit die Fehler dabei nicht zu groß werden, sollte man mit double-Werten 
// rechnen
double dx = 0.001;
double dy = 0.001;
double xStart = 0;
double yStart = 0;
int nOrbits = 1000; // Anzahl Orbits mit unterschiedlichem Radius
int nPoints = 4000; // Punkte pro Orbit

// Hilfsvariablen
int point = 0;
double xH = xStart;
double yH = yStart; 
PImage fbereich;

///////////////////////////////////////////////////////////////////////////

PImage zeichne_Fluchtbereich() {

 PImage img = createImage(width, height, ARGB);
  print("Berechne Fluchtbereich: ");
  for (int l = 0; l<width; l++) {
    for (int s =0; s<height; s++) {
      double xAlt = map(l, 0, width, xU, xO);
      double yAlt = map(s, 0, height, yO, yU);
      for (int k =0; k <= 100; k++) {
        double xNeu = xAlt*cos(alpha)-(yAlt-xAlt*xAlt)*sin(alpha); 
        double yNeu = xAlt*sin(alpha)+(yAlt-xAlt*xAlt)*cos(alpha); 
        xAlt = xNeu; 
        yAlt = yNeu;
      }
      double d = Math.sqrt(xAlt*xAlt + yAlt*yAlt);
      img.pixels[l+s*width] = ( d<10000) ? color(255, 255, 0) : color(0, 150, 150);
    }
    if ((l & 63) == 0) print(100*l/height + "% ");
  }
  xH = xStart; 
  yH = yStart;
  point = 0;

  return img;
}

///////////////////////////////////////////////////////////////////////////
//Initialisierungen  
///////////////////////////////////////////////////////////////////////////

void setup() { 
  size(800, 800);
  background(0);
}

///////////////////////////////////////////////////////////////////////////

void draw() {

  //Fraktal zeichnen
  iteriere(xH,yH); 
  point++;
  xH = xH + dx; 
  yH = yH + dy;
  if (point>=nPoints) {
    println("alle Punkte gesetzt, fertig"); 
    noLoop();
  }
  if (abs((float)xH) >= 3 && abs((float)yH) >=2) {
    println("äussere Grenze erreicht, fertig");
    noLoop();
  }
}

//////////////////////////////////////

void keyPressed() {  
  if (key == 'f') {
    if (fbereich==null) fbereich = zeichne_Fluchtbereich();
    image(fbereich, 0, 0);
    point=0;
  }  
  if (key == 'd') background(255);
  if (key == 's') save("image.png");
} 
/////////////////////////////////////////////////////
//Die Stelle, auf die man klickt wird vergrößert
//////////////////////////////////////////////////////

void mousePressed() {  
  background(255);
  float xUNeu = map(mouseX, 0, width, xU, xO)-(xO-xU)/5;
  float xONeu = map(mouseX, 0, width, xU, xO)+(xO-xU)/5;
  float yUNeu = map(mouseY, 0, height, yO, yU)-(yO-yU)/5;
  float yONeu = map(mouseY, 0, height, yO, yU)+(yO-yU)/5;
  xU = xUNeu; 
  xO = xONeu; 
  yU = yUNeu; 
  yO = yONeu;
  xH = xStart; 
  yH = yStart;
  nOrbits = nOrbits*3; 
  point = 0;
}

//////////////////////////////////////////////////////////

void iteriere(double xAlt,double yAlt) {  

  int i=0;
  stroke(color( (130+point*point)%256, (255-point)%256, point%256));
  while (i <=nOrbits) {
    if (Math.abs(xAlt)< 100000 && Math.abs(yAlt)< 100000) {
      double xNeu = xAlt*cos(alpha)-(yAlt-xAlt*xAlt)*sin(alpha); 
      double yNeu = xAlt*sin(alpha)+(yAlt-xAlt*xAlt)*cos(alpha);   
      float xK = round(map((float)xNeu, xU, xO, 0, width));                 
      float yK = round(map((float)yNeu, yU, yO, height, 0));    
      point(xK, yK);  
      xAlt = xNeu;
      yAlt = yNeu;
    }
    i++;
  }
}
